
from textual.app import ComposeResult, App
from textual.widgets import Static, Button, Input
from textual.screen import Screen, ModalScreen
from textual.containers import Grid
from .confmscn import Confirm_Screen
from textual import  on, work
from .revertpaxmodule import *
from .scan_serials import Scan_serials
from .functionsScreen import FunctionsScreen

class Select_QR_or_Barcode(ModalScreen[bool]):
       
    CSS_PATH = "css_lib/cofirm_screen.tcss"

    def compose(self) -> ComposeResult:
        yield Grid(
            Static("Please select input method:", id= "question"),
            Button("SCAN QR CODE", name="QR", id="qr"),
            Button("SCAN BARCODE", name='BC', id='BC'),
            id="confirmscreen"
        )
    
    @on(Button.Pressed, "#qr")
    def push_qr_screen(self) -> None:
        self.app.push_screen(Scan_qr())

    @on(Button.Pressed, "#BC")
    def push_bcScreen(self) -> None:
        self.app.push_screen(Scan_serials())

class Scan_qr(Screen):
    """QR SCANNER"""

    def compose(self) -> ComposeResult:
        yield Static("PlEASE SCAN QR CODE TO BEGIN", classes="question" )
        yield Input(placeholder=">>>>")
    
    @on(Input.Submitted)
    @work
    async def fix_qr(self) -> None:
        self.l = self.query_one(Input).value
        self.disabled = True
        self.serialList = eval(self.l)
        if await self.app.push_screen_wait(Confirm_Screen(f"Are these terminals you wish to activate\n{self.serialList}?")):
            self.notify("Activating>>>")

            apifun = apiPaxFunctions() 
            self.thing = await apifun.startPaxGroup(self.serialList)
            thing2 = await apifun.activateTerminals(apifun.idList)
            self.notify(str(thing2))
            self.app.push_screen(FunctionsScreen(self.thing))


class Input_app(App):

    def on_mount(self) -> None:
         self.push_screen(Select_QR_or_Barcode())
         




import asyncio
import aiohttp
import hmac
import time
import hashlib
from urllib.parse import urlencode, urlunsplit
import pandas as pd
import os



class PaxTerms():

    def __init__(self, serialNo:str) -> None:

    
        self.termtool = {
            "119": {"modelName": "E600", "serial_range":"119", "name":"TR_E600", "type":"Ev1_Series", "terminal_no":"12", "resellerName": "Repair","fmName":"E600_PayDroid_7.1.1_Virgo_V04.1.18_20240124", "hasAccessory":True, "accessory":
            {"modelName":"Q20L", "serial_range": "114", "name": "TR_E600-Q20", "type":"Qv1_Series", "terminal_no":"124", "resellerName": "Repair"}},
            "134": {"modelName": "E700", "serial_range":"134", "name":"TR_E700", "type":"Ev1_Series", "resellerName": "Repair","fmName": "E700_PayDroid_7.1.2_Scorpio_V10.1.29_20240320", "hasAccessory":True, "accessory":
            {"modelName":"Q20L", "serial_range": "114", "name": "TR_E700-Q20", "type":"Qv1_Series", "terminal_no":"124", "resellerName": "Repair"}},
            "115": {"modelName": "E500", "serial_range":"115", "name":"TR_E500", "type":"Ev1_Series", "terminal_no":"2", "resellerName": "Repair","fmName": "E500_PayDroid_6.0.1_Taurus_V05.1.25_20230313","hasAccessory":True, "accessory":
            {"modelName":"Q20L", "serial_range": "114", "name": "TR_E500-Q20", "type":"Qv1_Series", "terminal_no":"124", "resellerName": "Repair"}},
            "227": {"modelName": "E600M", "serial_range":"227", "name":"TR_E600M", "type":"Ev2_Series", "terminal_no":"415",  "resellerName": "Repair","fmName": "PayDroid_10.0.0_Acacia_V13.1.18_20240105","hasAccessory":True,"accessory":
            {"modelName":"Q10A", "serial_range": "240", "name": "TR_E600M-Q10", "type":"Qv2_Series", "terminal_no":"416", "resellerName": "Repair"}},
            "082": {"modelName": "A920", "serial_range":"082", "name":"TR_A920", "type":"A_Series", "terminal_no":"1", "resellerName": "Repair","fmName": "PayDroid_5.1.1_Aquarius_V02.3.46_20240410","hasAccessory":False},
            "185": {"modelName": "A920Pro", "serial_range":"185", "name":"TR_A920Pro", "type":"A_Series", "terminal_no":"341", "resellerName": "Repair","fmName": "PayDroid_8.1.0_Sagittarius_V11.1.62_20240411","hasAccessory":False},
            "135": {"modelName": "A60", "serial_range":"135", "name":"TR_A60", "type":"A_Series", "terminal_no":"19",  "resellerName": "Repair","fmName": "A60_PayDroid_6.0_Leo_V07.1.17_20240415","hasAccessory":False},
            "124": {"modelName": "A80", "serial_range":"124", "name":"TR_A80", "type":"A_Series", "terminal_no":"26", "resellerName": "Repair","fmName": "PayDroid_10.0_Cedar_V17.2.19_20240313", "hasAccessory":False},
            "176": {"modelName": "A77", "serial_range":"176", "name":"TR_A77", "type":"A_Series", "terminal_no":"271", "resellerName": "Repair","fmName": "PayDroid_8.1.0_Sagittarius_V11.1.62_20240411","hasAccessory":False},
            "3A4": {"modelName": "SP30s", "serial_range":"3A4", "name":"TR_SP30", "type":"A_Series", "terminal_no":"180", "resellerName": "Repair", "hasAccessory":False},
            "3A6": {"modelName": "SP30s", "serial_range":"3A6", "name":"TR_SP30", "type":"A_Series", "terminal_no":"180", "resellerName": "Repair", "hasAccessory":False},
            "3A7": {"modelName": "SP30s", "serial_range":"3A7", "name":"TR_SP30", "type":"A_Series", "terminal_no":"180", "resellerName": "Repair", "hasAccessory":False},
            "3L0": {"modelName": "SP30s", "serial_range":"3L0", "name":"TR_SP30", "type":"A_Series", "terminal_no":"180", "resellerName": "Repair", "hasAccessory":False},
            "229": {"modelName": "A35", "serial_range":"229", "name":"RKI_A35", "type":"A_Series", "terminal_no":"362", "resellerName": "Repair","fmName": "PayDroid_10.0_Cedar_V17.2.19_20240313", "hasAccessory":False},
            "114": {"modelName": "Q20L", "serial_range":"114","name":None, "type":"Qv1_Series", "hasAccessory":False, "resellerName":"Repair", "fmName":None},
            "240": {"modelName": "Q10A", "serial_range":"240","name":None, "type":"Qv2_Series","fmName": "Q10A_PayDroid_10_Cedar_V17.1.13_20240322","resellerName":"Repair", "hasAccessory":False},
            "189": {"modelName": "D135", "serial_range":"189", "name":"TR_D135", "type":"D-Series", "terminal_no":"421", "resellerName": "Repair", "hasAccessory":False},
        }
        
        self.pushPATool = {
                "A_SeriespackageName": "com.pax.us.pay.std.broadpos.p2pe",
                "A_SeriespushTemplateName": "PA 6.8 A-Series BroadPos P2PE V1.05.05",
                "A_SeriestemplateName":  "p2pe_20231218.zip",
                "A_Seriesversion": "V1.05.05_20231218",
                "Qv2_SeriespackageName": "com.pax.us.pay.std.broadpos.p2pe",
                "Qv2_SeriespushTemplateName": "PA 6.9.1 E600M BroadPos P2PE V1.05.06",
                "Qv2_SeriestemplateName": "p2pe_20240325.zip",
                "Qv2_Seriesversion": "V1.05.06_20240325",
                "Qv1_SeriespackageName":"BroadPOS-P2PE-Q20L",
                "Qv1_SeriespushTemplateName": "RPA 6.8 Q20 BroadPOS P2PE Q20L V1.01.05_20230413",
                "Qv1_SeriestemplateName": "config.zip",
                "Qv1_Seriesversion": "V1.01.05_20230413",
                "Ev1_SeriespackageName": "com.pax.pdm",
                "Ev1_SeriestemplateName": "BasicSystem-Q20L_V1.00.07.zip",
                "Ev1_Seriesversion": None,
                "Ev1_SeriespushTemplateName": None,
                "Ev2_SeriespackageName": None,
                "Ev2_SeriestemplateName": None,
                "Ev2_Seriesversion": None,
                "Ev2_SeriespushTemplateName": None
        }

        
        
        self.serialNo = serialNo
        for k,v in self.termtool.items():
            if str(self.serialNo)[0:3] == k in self.termtool:
                self.modelName = v["modelName"]
                self.name = v["name"]
                self.resellerName = v["resellerName"]
                self.type = v["type"]
                self.fmName = v["fmName"]
                if v["hasAccessory"]: 
                    self.hasAccessory = True
                    self.accessoryModelName = v["accessory"]["modelName"]
                    self.accessoryName = v["accessory"]["name"]
                    self.accessoryResellerName = v["accessory"]["resellerName"]
                    self.accessoryType = v["accessory"]["type"]
                else: 
                    self.hasAccessory = False
                    self.accessoryModelName = None
                    self.accessoryName = None
                    self.accessoryResellerName = None
                    self.accessoryType = None

        self.bPosPackageName = self.pushPATool[f"{self.type}packageName"]
        self.bPosPushTemplateName = self.pushPATool[f"{self.type}pushTemplateName"]
        self.bPosTemplateName = self.pushPATool[f"{self.type}templateName"]
        self.bPosPackageVersion = self.pushPATool[f"{self.type}version"]


   
                
class PaxApiTool():
    """Creates aiohttp.ClientSession request object to interface with the public PaxStore API.
    \nRequired parameters: apiKey, apiSecret, session (aiohttp.ClientSession), operation
    \nPossilbe operations: 'createTerminal', 'findTerminal','activateTerminal', 'disableTerminal', 'moveTerminal'
    """
    def __init__(self, apiKey, apiSecret, session:aiohttp.ClientSession,operation:str,terminalId:str|None=None, serialNo = None, merchantName = None, modelName = None, name = None, resellerName = None, status = None, command = None, packageName = None, pushTemplateName=None, templateName=None, version=None, rkiKey=None, fmName=None, list=None) -> None:

        self.apiKey = apiKey
        self.apiSecret = apiSecret
        self.baseUrl = "https://api.paxstore.us"
        self.altbaseUrl = "https://api.whatspos.com"
        self.scheme = "https"
        self.netloc = "api.paxstore.us"
        self.altnetloc =  'api.whatspos.com'
        self.session = session
        self.operation = operation
        self.terminalId = terminalId
        self.serialNo = serialNo
        self.merchantName = merchantName
        self.modelName = modelName
        self.name = name
        self.resellerName = resellerName
        self.status = status

        self.command = command
        self.packageName = packageName
        self.pushTemplateName = pushTemplateName
        self.templateName = templateName
        self.version = version
        self.rkiKey = rkiKey
        self.fmName = fmName
        self.list = list
        self.query = {
            "sysKey": self.apiKey,
            "timeStamp": round(time.time()*1000)
        }
        if self.list:
            self.unpacked = str(f'{", ".join(self.list)}')
        else:
            self.unpacked = None
        self.operations_dict = {
            "createTerminal": {
                "path": "/p-market-api/v1/3rdsys/terminals",
                "method": self.session.post,
                "addQuery": None,
                "body": {
                    "merchantName":self.merchantName,
                    "modelName":self.modelName,
                    "name":self.name,
                    "resellerName":self.resellerName,
                    "serialNo":self.serialNo,
                    "status":self.status
                }
                },
            "findTerminal": {
                "path": "/p-market-api/v1/3rdsys/terminals",
                "method": self.session.get,
                "addQuery": {
                    "snNameTID": self.serialNo,
                },
                "body": None
            },
            "activateTerminal": {
                "path": f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}/active",
                "method": self.session.put,
                "addQuery": None,
                "body": None
            },
            "disableTerminal" : {
                "path": f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}/disable",
                "method": self.session.put,
                "addQuery": None,
                "body": None
            },
            "moveTerminal" : {
                "path": f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}/move",
                "method": self.session.put,
                "addQuery": None,
                "body": {
                    "merchantName": self.merchantName,
                    "resellerName": self.resellerName
                }
            },
            "updateTerminal" : {
                "path" : f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}",
                "method" : self.session.put,
                "addQuery": None,
                "body": {
                    "name": self.name,
                    "modelName": self.modelName,
                    "resellerName": self.resellerName,
                    "tid": "",
                    "serialNo": self.serialNo,
                    "merchantName": self.merchantName
                }
            },
            "deleteTerminal" : {
                "path" : f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}",
                "method": self.session.delete,
                "addQuery": None,
                "body": None
            },
            "terminalDetails" : {
                "path" : f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}",
                "method": self.session.get,
                "addQuery" : {
                    "includeDetailInfo" : "true",
                },
                "body": None
            },
            "accessoryDetails" : {
                "path" : f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}",
                "method": self.session.get,
                "addQuery" : {
                    "includeDetailInfo": "true",
                    "includeDetailInfoList" : "true"
                },
                "body": None
            },
            "pushCommand": {
                "path": f"/p-market-api/v1/3rdsys/terminals/{self.terminalId}/operation",
                "method": self.session.post,
                "addQuery": {
                    "Command": self.command
                },
                "body": None
            },
            "pushParamAPK": {
                "path": "/p-market-api/v1/3rdsys/terminalApks",
                "method": self.session.post,
                "addQuery": None,
                "body": {
                    "packageName": self.packageName,
                    #"pushTemplateName": self.pushTemplateName,
                    "serialNo": self.serialNo,
                    "templateName": self.templateName,
                    "version": self.version
                }
            },
            "pushAPK": {
                "path": "/p-market-api/v1/3rdsys/terminalApks",
                "method": self.session.post,
                "addQuery": None,
                "body": {
                    "packageName": self.packageName,
                    "serialNo": self.serialNo
                }  
            },
            "pushRKI": {
                "path": "/p-market-api/v1/3rdsys/terminalRkis",
                "method": self.session.post,
                "addQuery": None,
                "body": {
                    "rkiKey": self.rkiKey,
                    "serialNo": self.serialNo
                }
            },
            "pushFirmware": {
                "path": "/p-market-api/v1/3rdsys/terminalFirmwares",
                "method": self.session.post,
                "addQuery": None,
                "body": {
                    
                    "fmName": self.fmName,
                    "serialNo": self.serialNo
                }
            },
            "terminalConfig":{
                "path":"/p-market-api/v1/3rdsys/terminals",
                "method":self.session.get,
                "addQuery": {
                    "includeInstalledApks":"true",
                    "includeInstalledFirmware": "true",
                    "pageNo":"",
                    "pageSize":"",
                    "snNameTID": self.serialNo,

                },
                "body": None
            },
            "appPushHistory": {
                "path": "/p-market-api/v1/3rdsys/parameter/push/history",
                "method":self.session.get,
                "addQuery": {
                    "onlyLastPushHistory": "false",
                    "packageName": self.packageName,
                    "pageNo": 1,
                    "pageSize": 20,
                    "pushStatus":3,
                    "serialNo": self.serialNo
                },
                "body": None
            }

        }

        for k, v in self.operations_dict.items():
            if self.operation == k in self.operations_dict.keys():
                self.path = v["path"]
                if v["addQuery"]:
                    self.query.update(v["addQuery"])
                if v["body"]:
                    self.body = v["body"]
                else: self.body = None
                self.method = v["method"]
                self.encodedQuery = urlencode(self.query)
                self.fullurl = urlunsplit((self.scheme,self.netloc,self.path,self.encodedQuery,None))
                #print(self.fullurl)
                self.signature = hmac.new(self.apiSecret, self.encodedQuery.encode('utf-8'), hashlib.md5).hexdigest().upper()





#operation:str, terminalId = None, serialNo = None, merchantName = None, modelName = None, name = None, resellerName = None, status = None, command = None

#terminalId = None, serialNo = None, merchantName = None, modelName = None, name = None, resellerName = None, status = None, command = None
async def buildRequest(session,operation:str,**kwargs):
    
    apiKey = os.environ.get("APIKEY").encode('utf-8')
    apiSecret = os.environ.get("APISECRET").encode("utf-8")
    term = PaxApiTool(apiKey,apiSecret,session,operation, **kwargs)
    headers = {"Content-Type": "application/json; charset=UTF-8","signature":term.signature}
    method = term.method
    print(term.body)
    print(term.fullurl)
    async with method(term.fullurl,json=term.body,headers=headers) as resp:
        term_data = await resp.json(content_type=None)
    return term_data


async def findTerminal(serialNoList: list) -> list:

    """
Purpose:
Locates a PaxStore terminal based on its serial number.
   
Parameters: serialNoList (list): A list of serial numbers to search for.

Returns: list: A list of JSON responses, each containing information about a found terminal. If a terminal is not found, its corresponding response will be an empty list."""
    
    async with aiohttp.ClientSession() as session:
        tasks = []
        for serialNo in serialNoList:
            tasks.append(asyncio.ensure_future(buildRequest(session, "findTerminal", serialNo = serialNo)))
        responses = await asyncio.gather(*tasks)
    cleanResponses = []
    for resp in responses:
        #print(resp)
        cleanResponses.append(*resp['dataset'])
    return cleanResponses

async def terminalDetails(idList: list, serialNoList=None, df=None) -> list:
    """
    Retrieves detailed information for a list of terminals.

    Args:
        id_list (list): A list of terminal IDs.
        serial_no_list (list, optional): A list of serial numbers (alternative to id_list).
        df (pd.DataFrame, optional): A DataFrame containing terminal information.

    Returns:
        list: A list of JSON responses, each containing detailed information about a terminal.
    """
    
    async with aiohttp.ClientSession() as session:
        tasks = [asyncio.create_task(buildRequest(session, "terminalDetails", terminalId=id)) for id in idList]
        responses = await asyncio.gather(*tasks)
        cleanResponses = []
    for resp in responses:
        #print(resp)
        cleanResponses.append(resp['data']['terminalDetail'])
        #print(cleanResponses)

    return cleanResponses
            
async def accessoryDetails(df:pd.DataFrame):
    """tasks = []
    accessoryList = []
    print(df)
    idList = df['id'].to_list()"""

    async with aiohttp.ClientSession() as session:
        
        tasks = [asyncio.ensure_future(buildRequest(session, "accessoryDetails", terminalId=id)) for id in df["id"]]
        responses = await asyncio.gather(*tasks)

        """for resp in responses:
            #print(resp)
            if "terminalAccessory" in resp['data'].keys():
                basic = resp['data']['terminalAccessory']['basic']
                accessoryList.append(str(*[dic.get("content") for dic in basic if dic.get('name')=="SN"]))
            else: accessoryList.append(None)"""
        accessoryList = [resp["data"]["terminalAccessory"]["basic"][0]["content"] if "terminalAccessory" in resp["data"] else None for resp in responses]

        qf = pd.DataFrame({"accessory":accessoryList}, dtype=object)
        ndf = pd.concat([df,qf],axis=1)
        print(ndf)
    #print("\n\n\n\******",accessoryList)
    #print("________________\n\n",copyAccList)
    
    accessoryid = await findTerminal(ndf['accessory'].dropna())
    mqf = pd.DataFrame(accessoryid,dtype=object)
    print(mqf)
    bqf = pd.concat([ndf,mqf], ignore_index=True)
    print(bqf)
    #print("\n\n\n******",accessoryid) 
    print(accessoryid)
    return bqf
    

class apiPaxFunctions(): 

    def __init__(self) -> None:
        pass

    
    
    async def startPaxGroup(self, serialNoList):
        self.idList = []
        self.serialNoList = serialNoList
        findResp = await findTerminal(serialNoList)
        fullserial_List = []
        for resp in findResp:   
            print("\n\n\nRESP:",resp)
            self.idList.append(resp['id'])
        detailresp = await terminalDetails(self.idList)




        print("\n\n\nfindResp:",*findResp)
        self.groupList = []
        for fresp,dresp in list(zip(findResp,detailresp)):
            fresp.update(dresp)
            self.groupList.append(fresp)
        #for thing in self.groupList:
            #print("\n\n\nThing:",thing)
        group_df = pd.DataFrame(self.groupList, dtype=object)
        accessoryDetail = await accessoryDetails(group_df)
        #print("\n\n\nAcessoryDetail",accessoryDetail)
        

        #print(self.serialNoList,self.idList)
        return accessoryDetail
    
    async def activateTerminals(self, idList, serialNoList=None, df=None):
        
        tasks = []
        responses = []

        async with aiohttp.ClientSession() as session:
            for id in idList:
                tasks.append(asyncio.ensure_future(buildRequest(session,"activateTerminal",terminalId = id)))
            result = await asyncio.gather(*tasks)
        for res in result:
            if res == None:
                responses.append({'businessCode':0, 'message':'The terminal has been activated'})
            else: responses.append(res)   
        return responses

    async def disableTerminals(self, idList:list, serialNoList=None, df=None):

        tasks = []
        responses = []
        async with aiohttp.ClientSession() as session:
            for id in idList: 
                tasks.append(asyncio.ensure_future(buildRequest(session,'disableTerminal', terminalId = id)))
            result = await asyncio.gather(*tasks)
        for res in result:
            if res == None:
                responses.append({'businessCode':0, 'message':'The terminal has been disabled'})
            else: responses.append(res)   
        return responses
    
    async def createTerminals(self,serialNoList):

        tasks = []
        cleanResponses = []
        async with aiohttp.ClientSession() as session:
            for serialNo in serialNoList: 
                terminal = PaxTerms(serialNo)
                tasks.append(asyncio.ensure_future(buildRequest(serialNo=serialNo,session=session, operation="createTerminal",merchantName="North American Bancard",name=terminal.name,modelName=terminal.modelName,resellerName=terminal.resellerName,status = "A")))
            result = await asyncio.gather(*tasks) 
            for resp in result:
                cleanResponses.append(resp['data'])
            #cleanResponses.append(*[resp['data'] for resp in result])
            return cleanResponses

    async def deleteTerminals(self, idList, df=None):
        tasks = []
        async with aiohttp.ClientSession() as session:
            for id in idList:
                tasks.append(asyncio.ensure_future(buildRequest(session, "deleteTerminal", terminalId = id )))
            result = await asyncio.gather(*tasks)

            return result
        

        
    
    async def pushTerminalAPK(self, serialNoList, paramApp:bool=None, packageName=None):
        tasks = []
        async with aiohttp.ClientSession() as session:
            for serialNo in serialNoList: 
                if paramApp:
                    terminal = PaxTerms(serialNo)
                    tasks.append(asyncio.ensure_future(buildRequest(session, "pushParamAPK",serialNo=serialNo,packageName=terminal.bPosPackageName,templateName=terminal.bPosTemplateName,version=terminal.bPosPackageVersion)))
                else:
                    tasks.append(asyncio.ensure_future(buildRequest(session, "pushAPK",serialNo=serialNo, packageName=packageName)))
            result = await asyncio.gather(*tasks)
            return result

    async def pushThingy(self, terminalList ,operation:str, **kwargs):
        """Pushes thing to group of terminals. """
        tasks = []
        async with aiohttp.ClientSession() as session:
            for serialNo in terminalList:
                tasks.append(asyncio.ensure_future(buildRequest(session,operation, serialNo=serialNo, **kwargs)))
            result = await asyncio.gather(*tasks)
            return result
        
async def notnain():
    tasks=[]
    id_list = []
    task2 = []
    
    serial_list = input('Please scan QR:\n>>>>')
    async with aiohttp.ClientSession() as s:
        for serial in serial_list:   
            terminal = PaxTerms(serial)
            tasks.append(asyncio.ensure_future(buildRequest(serialNo=serial, session = s, operation="createTerminal",merchantName="North American Bancard",name=terminal.name,modelName=terminal.modelName,resellerName=terminal.resellerName,status = "P")))
        res = await asyncio.gather(*tasks)
        print(res)

class PushConfigs():
    
    def __init__(self) :
        pass

        

    async def pushPAConfig(self, serialNoList, idList = None, df = None):
        """Pushes current version of:\n BroadPosP2PE (PA Template),\n CheckUp,\n PAInstaller, \n PayDroid Firmware, \n RKI"""
        task = apiPaxFunctions()
        #bpospush = await task.pushTerminalAPK(serialNoList,True)
        #print("BroadPosP2PEPush:",bpospush)
        installerPush = await task.pushTerminalAPK(serialNoList,False, "com.nabancard.painstaller")
        print("PAInstallerPush:", installerPush)
        checkupPush = await task.pushTerminalAPK(serialNoList,False,"com.pax.checkup")
        print("CheckUpPush",checkupPush)
        rki = await task.pushThingy(serialNoList,"pushRKI",rkiKey="EPX_PIN_Slot1_Data_Slot3")
        print(rki)

        

    async def pushBroadPosEPX(self, serialNoList,idList = None, df = None):
        
        
        task = apiPaxFunctions()
        checkupPush = await task.pushTerminalAPK( serialNoList,False,"com.pax.checkup")
        print("CheckUpPush",checkupPush)
        rki = await task.pushThingy(serialNoList,"pushRKI",rkiKey="EPX_PIN_Slot1_Data_Slot3")
        print(rki)


    async def pushBroadPos_nonEPX(self, serialNoList,idList = None, df = None):
        
        task = apiPaxFunctions()
        checkupPush = await task.pushTerminalAPK(serialNoList,False,"com.pax.checkup")
        print("CheckUpPush",checkupPush)
        





async def main():
    

    serialNoList = ['0822776026', '0821674870', '0821256789']
    for serialNo in serialNoList:
        term = PaxTerms(serialNo)
        print(term.modelName,term.hasAccessory,term.accessoryModelName)
    thing = apiPaxFunctions()    
    thing2 = await findTerminal(serialNoList)
    print(thing2)
    
    #q20List = await accessoryDetails(thing.idList)
    
    



        


    

    
if __name__=='__main__':
    asyncio.run(main())






from pathlib import Path
from typing import Iterable
import time
from textual.app import App, ComposeResult
from textual.widgets import DirectoryTree, Static, Label, Button, LoadingIndicator
from textual.screen import Screen, ModalScreen
from textual.containers import Container, Horizontal, VerticalScroll, Grid
from textual import events, on, work
from textual.reactive import reactive
import pandas as pd
from DFTable import DataFrameTable
from textual.message import Message
import os
import datetime

def read_xl(file):
    """Imports Excel spreadsheet"""

    Erequire_cols = [0]
    df = pd.read_excel(file, skiprows=[0], usecols=Erequire_cols,index_col=None,dtype=object)
    serial_list = df['Serial No'].astype("str").values
    tid_list = get_term_ids_v2(list(serial_list)) # type: ignore
    tid_list.to_excel("new_file2.xlsx")

    return tid_list

class PathLabel(Label):
    path: reactive[str] = reactive("None")

    def render(self) -> str:
        return f"Selected path: {self.path}"

class GroupLabel(Label):
    group: reactive [str] = reactive("None")

    def render(self) -> str:
        return f"Selected group: {self.group}"

class FilePicker(DirectoryTree):
    
    def filter_paths(self, paths: Iterable[Path]) -> Iterable[Path]:
        return [path for path in paths if not path.name.startswith(".")] 

class GroupButton(Button):
    
    """Button For PaxGroups"""

    class Selected(Message):

        def __init__(self,group:str) -> None:
            self.group = group
            super().__init__()

    def __init__(self,group:str) -> None:
        self.group = group
        super().__init__()
    
    def on_mount(self) -> None:
        self.styles.margin = (1, 2)
        self.styles.content_align = ("center", "middle")

    def on_button_pressed(self) -> None:
        self.post_message(self.Selected(self.group))
    
    
    
    def render(self) -> str:
        return str(self.group)



class Confirm_Screen(ModalScreen[bool]):
    CSS_PATH = "src/textual_pax/css_lib/cofirm_screen.tcss"

    def __init__(self, message:str):
        self.message = message
        super().__init__()

    def compose(self) -> ComposeResult:
        yield Grid(
            Static(self.message, id="question"),
            Button("Cancel", id="cancel", variant="error"),
            Button("OK", id="ok", variant="success")
            ,id="confirmscreen"
        )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "ok":
            self.dismiss(True)
        else:
            self.dismiss(False)

class GroupSelection(Screen):

    def __init__(self, question) -> None:
        self.question = question
        super().__init__()

    def compose(self) -> ComposeResult:
        yield Static(self.question)
        yield Button("Group 1", id="Group_1")
        yield Button("Group 2", id="Group_2")
        yield Button("Group 3", id="Group_3")
        yield Button("Group 4", id="Group_4")
        yield Button("Group 5", id="Group_5")
        yield Button("Group 6", id="Group_6")

    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.dismiss(str(event.button.id))

     

class PaxGroup(Screen):

    def __init__(self,file: str) -> None:
        self.file = file
        super().__init__()

    def compose(self) -> ComposeResult:
        yield DataFrameTable()
        yield Static("CREATE PAX GROUP?")
        yield Button("No", id="no", variant="error")
        yield Button("Yes", id="yes", variant="success")
   
    def on_mount(self):
        """Imports Excel spreadsheet"""
        Erequire_cols = [0]
        df = pd.read_excel(self.file, skiprows=[0], usecols=Erequire_cols,index_col=None,dtype=object)
        serial_list = df['Serial No'].astype("str").values
        self.tid_list = get_term_ids_v2(list(serial_list)) # type: ignore
        self.model_list = self.tid_list["label"]
        table = self.query_one(DataFrameTable)
        table.add_df(self.tid_list)
        
        self.loading = False
        self.tid_list.to_excel("again.xlsx")

        
    
    def handle_confirm(self, cancel:bool):
        """Callback for ConfirmScreen"""
        if cancel:
            self.dismiss()

    def accept_answer(self, group:str): 
        """Callback for GroupSelection screen"""
        self.notify(group)
    
        try:
            Current_Date = str(datetime.datetime.today().strftime('%m.%d.%Y.%H.%M'))
            path = f"Iface_Pax_Groups/{group}"
            os.makedirs(path)
            os.chdir(path)
            file_name = str(self.model_list[0]+Current_Date+group.removeprefix("Group_")+".pkl")
            
            self.tid_list.to_pickle(file_name)
            
        except OSError:
            self.app.push_screen(Confirm_Screen(message=f"{group} FILE EXISTS\nOverwirite Existing Group?"),callback=self.handle_confirm)


    @on(Button.Pressed, "#yes")
    #@work
    def choose_group_id(self, group:str):
        self.app.push_screen(GroupSelection("Please Assign Group ID"),self.accept_answer)

class Selector_Widget_Screen(Screen):

    def compose(self) -> ComposeResult:
        
        
        yield Static("PLEASE SELECT FILE TO IMPORT")
        yield FilePicker(str(os.curdir))
        yield PathLabel()
        

    @on(FilePicker.FileSelected)
    @work
    async def show_path(self, event: DirectoryTree.FileSelected) -> None:
        self.notify(str(event.path))
        if await self.app.push_screen_wait(Confirm_Screen(message="IS THIS THE FILE TO UPLOAD?")): #type: ignore
            self.app.query_one(PathLabel).path = str(event.path)
            self.app.push_screen(PaxGroup(str(event.path)))
        else: self.app.query_one(PathLabel).path = str(None)
    
    def action_confirm_file(self):
        """confirm file"""
        def use_file(use: bool):
            if use:
                self.app.pop_screen()

class Selector(App):
    
    def on_mount(self) -> None:
        self.push_screen(Selector_Widget_Screen())

if __name__ == "__main__":
    app = Selector()
    app.run()





from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.units import mm
from reportlab.graphics.barcode import createBarcodeDrawing
from reportlab.graphics.shapes import Drawing, String, Group
from reportlab.graphics import renderPDF
import datetime
import pandas as pd 
import asyncio


"""PAGESIZE = (89*mm,28*mm)
PAGEWIDTH = PAGESIZE[1]
PAGEHEIGHT = PAGESIZE[0]
TEXT_Y = 55
NUM_LABELS_X = 1
NUM_LABELS_Y = 1
LABEL_WIDTH = PAGESIZE[0] / NUM_LABELS_X
LABEL_HEIGHT = PAGESIZE[1] / NUM_LABELS_Y
SHEET_TOP = PAGESIZE[1]
BARCODE_Y = 50"""

class PDFLabels():

    """
    Represents a collection of labels on a PDF page.

    Attributes:
        num_labels_x (int): Number of labels horizontally.
        num_labels_y (int): Number of labels vertically.
        text_y (int): Vertical position of text (in points).
        barcode_y (int): Vertical position of barcode (in points).
        pagesize (tuple): Page size (width, height) in points.
        labelwidth (float): Width of a single label (in points).
        labelheight (float): Height of a single label (in points).
        sheettop (float): Top position of the label sheet (in points).
        u (range): Range of y-axis indices.
        i (range): Range of x-axis indices.
    """


    def __init__(self, pagesize:tuple, num_labels_x:int, num_labels_y:int, text_y:int, barcode_y:int) -> None:
        
        self.pagesize = pagesize
        self.num_labels_x = num_labels_x
        self.num_labels_y = num_labels_y
        self.text_y = text_y
        self.barcode_y = barcode_y
        self.pagewidth = self.pagesize[1]
        self.pageheight = self.pagesize[0]
        self.labelwidth = self.pagesize[0]/self.num_labels_x
        self.labelheight = self.pagesize[1]/self.num_labels_y
        self.sheettop = self.pagesize[1]
        self.u = range(0,self.num_labels_y)
        self.i = range(0,self.num_labels_x)

    def draw_it(self, c: Canvas, label_drawing: Drawing):  
        """
        Draws the label layout onto the specified canvas.

        Args:
            c (Canvas): The PDF canvas.
            label_drawing (Drawing): The label drawing object.
        """
        for u in range(0, self.num_labels_y):
            for i in range(0, self.num_labels_x):
                x = i * self.labelwidth
                y = self.sheettop - self.labelheight - u * self.labelheight
                renderPDF.draw(label_drawing, c, x, y)

    def map_it(self) -> list[tuple[int, int]]:
        """
        Returns a list of (x, y) coordinates for all label positions.

        Returns:
            list[tuple[int, int]]: A list of coordinate tuples.
        """        

        return [(i * self.labelwidth, self.sheettop - self.labelheight - u * self.labelheight)
            for u in range(self.num_labels_y)
            for i in range(self.num_labels_x)]
    
    def create_label(self, model: str, model_pn: str, serial_no: str, iccid: str = None) -> Drawing:
        barcode = createBarcodeDrawing("Code128", value=serial_no, width=45 * mm, height=6 * mm)
        x0, y0, bw, bh = barcode.getBounds()  # type: ignore
        barcode_x = (self.labelwidth - bw) / 2  # center barcode
        barcode_y = self.barcode_y  # spacing from label bottom (pt)
        text = String(0, self.barcode_y - 7, f"PAX {model} {model_pn}", fontName="Helvetica-Bold", fontSize=6, textAnchor="middle")
        text.x = self.labelwidth / 2  # type: ignore
        label_drawing = Drawing(self.pagewidth, self.pageheight)  # type: ignore
        label_drawing.add(text)
        if iccid:
            barcode_2 = createBarcodeDrawing("Code128", value=iccid, width=55 * mm, height=6 * mm)
            bc2x0, bc2y0, bc2bw, bc2bh = barcode_2.getBounds()  # type: ignore
            bc_2_x = (self.labelwidth - bc2bw) / 2
            text_2 = String(0, self.barcode_y - 37, "AT&T SIM: " + iccid, fontName="Helvetica-Bold", fontSize=6, textAnchor="middle")
            text_2.x = self.labelwidth / 2  # type: ignore
            label_drawing.add(barcode_2)
            label_drawing.add(text_2)
        label_drawing.add(barcode, barcode_x, barcode_y)

    def label_w_ICCID(self,model_pn: str, serial_no: str, iccid_no: str):

        barcode = createBarcodeDrawing("Code128", value=serial_no, width=45*mm, height=6*mm)
        x0, y0, bw, bh = barcode.getBounds() # type: ignore
        print(barcode.getBounds()) # type: ignore
        barcode_x = (self.labelwidth - bw) / 2  # center barcode
        barcode_y = self.barcode_y  # spacing from label bottom (pt)
        text = String(0, self.barcode_y - 7, model_pn+" S/N: "+serial_no, fontName="Helvetica-Bold", fontSize=6, textAnchor="middle")
        text.x = self.labelwidth / 2 # type: ignore
        bc_2 = createBarcodeDrawing("Code128", value=iccid_no, width=55*mm, height=6*mm)
        bc2x0, bc2y0, bc2bw, bc2bh = bc_2.getBounds() # type: ignore
        bc_2_x = (self.labelwidth - bc2bw) / 2
        text_2 = String(0, self.barcode_y - 37, "AT&T SIM: "+iccid_no, fontName="Helvetica-Bold", fontSize=6, textAnchor="middle")
        text_2.x = self.labelwidth / 2 # type: ignore
        label_drawing = Drawing(self.pagewidth,self.pageheight)# type: ignore
        bc_g = Group()
        bc_g2 = Group(bc_2)
        bc_g.add(barcode)
        bc_g.shift(barcode_x,barcode_y)
        bc_g2.shift(bc_2_x,barcode_y-30)
        label_drawing.add(bc_g)
        label_drawing.add(bc_g2)
        label_drawing.add(text)
        label_drawing.add(text_2)
        return label_drawing

    def simple_label(self,model: str, model_pn: str, serial_no: str):
        text = String(0, self.text_y, model, fontName="Helvetica-Bold", fontSize=12,textAnchor="middle")
        text.x = self.labelwidth / 2 # type: ignore
        barcode = createBarcodeDrawing("Code128", value=serial_no, width=69*mm, height=10*mm)
        x0, y0, bw, bh = barcode.getBounds() # type: ignore

        barcode_x = (self.labelwidth - bw) / 2  # center barcode
        barcode_y = self.barcode_y  # spacing from label bottom (pt)

        text_2 = String(0, barcode_y - 10, str(model_pn)+" S/N: "+str(serial_no), fontName="Helvetica", fontSize=6, textAnchor="middle")
        text_2.x = self.labelwidth / 2 # type: ignore
        label_drawing = Drawing(self.pagewidth,self.pageheight) # type: ignore
        bc_g = Group()
        bc_g.add(barcode)
        bc_g.shift(barcode_x,barcode_y)
        label_drawing.add(text)
        label_drawing.add(bc_g)
        label_drawing.add(text_2)
        
        return label_drawing     

async def create_pdf(df:pd.DataFrame, group=None, **kwargs) ->str:
    """
    Creates a PDF document containing labels for devices in the provided DataFrame.

    Args:
        df (pd.DataFrame): DataFrame containing device information.
        group (str, optional): Group identifier to include in the filename. Defaults to None.

    Returns:
        str: Message indicating successful PDF creation and filename.
    """

    filteredDataFrame = df.drop(df[(df.modelName=="Q20L")|(df.modelName=="Q10A")].index) # Filter out specific models
    # Extract data for label generation
    serial_list = filteredDataFrame['serialNo']
    mod_list = filteredDataFrame['modelName']
    wifi_mac_list = filteredDataFrame['macAddress']
    q_20_serial_list = filteredDataFrame['accessory']
    try: 
        iccid_list = filteredDataFrame['iccid']
    except KeyError:
        iccid_list = ["nan" for serial in serial_list]
    try:     
        mod_pn_list = filteredDataFrame['pn']
    except KeyError:
        if mod_list[0] == "SP30s": 
            mod_pn_list = ['SP30-00L-264-03E0']


        
    print(q_20_serial_list)   
    full_serial_list = [f"{serial}-{qserial}" if str(qserial) not in {"nan", "None"} else serial for serial, qserial in zip(serial_list, q_20_serial_list)]
    print(full_serial_list)
    current_date = str(datetime.datetime.today().strftime('%m.%d.%Y.%H.%M'))
    # Generate filename
    filename = (f"{mod_list[0]}Labels_{current_date}_G{group}" if group else f"{mod_list[0]}Labels_{current_date}_SN{serial_list[0]}.pdf")
    c = Canvas(filename, pagesize=(89*mm,28*mm))
    for mod,mod_pn, serial, iccid in list(zip(mod_list,mod_pn_list,full_serial_list, iccid_list)):
        if str(iccid) == 'nan':
            new_label = PDFLabels((89*mm,28*mm),1,1,60,25) # type: ignore
            sticker = new_label.simple_label(f'PAX {mod}', mod_pn,serial)
            new_label.draw_it(c,sticker)
        elif str(iccid) !='nan':
            new_label = PDFLabels((89*mm,28*mm),1,1,55,50) # type: ignore
            sticker = new_label.label_w_ICCID(mod_pn,serial, iccid)
            new_label.draw_it(c,sticker)
        c.showPage()
    print(f'There are {(len(iccid_list))} Pages in this document')
    c.save()

    return f"Labels Created \n Filename: {filename} \n Pages: {(len(full_serial_list))}"

"""df = pd.read_pickle("df.pkl")
asyncio.run(create_pdf(df,None))
"""



from textual.app import App, ComposeResult
from textual.widgets import Static, Button, Placeholder, Input, Label, TextArea, Header,Footer, MarkdownViewer
from textual.screen import Screen, ModalScreen
from textual.containers import Container, Horizontal, VerticalScroll, Grid
from confmscn import Confirm_Screen
from textual import events, on, work
import pandas as pd
from revertpaxmodule import *
from DFTable import DataFrameTable


serialNo = "082214439"

EXAMPLE_MARKDOWN = f"""\
# {serialNo}

This is an example of Textual's `MarkdownViewer` widget.


## Features

Markdown syntax and extensions are supported.

- Typography *emphasis*, **strong**, `inline code` etc.
- Headers
- Lists (bullet and ordered)
- Syntax highlighted code blocks
- Tables!

## Tables

Tables are displayed in a DataTable widget.

| Name            | Type   | Default | Description                        |
| --------------- | ------ | ------- | ---------------------------------- |
| `show_header`   | `bool` | `True`  | Show the table header              |
| `fixed_rows`    | `int`  | `0`     | Number of fixed rows               |
| `fixed_columns` | `int`  | `0`     | Number of fixed columns            |
| `zebra_stripes` | `bool` | `False` | Display alternating colors on rows |
| `header_height` | `int`  | `1`     | Height of header row               |
| `show_cursor`   | `bool` | `True`  | Show a cell cursor                 |


## Code Blocks

Code blocks are syntax highlighted, with guidelines.

```python
class ListViewExample(App):
    def compose(self) -> ComposeResult:
        yield ListView(
            ListItem(Label("One")),
            ListItem(Label("Two")),
            ListItem(Label("Three")),
        )
        yield Footer()
```
"""


class MarkdownExampleApp(App):
    def compose(self) -> ComposeResult:
        yield MarkdownViewer(EXAMPLE_MARKDOWN, show_table_of_contents=True)


if __name__ == "__main__":
    app = MarkdownExampleApp()
    app.run()




from textual.app import App, ComposeResult
from textual.widgets import DirectoryTree, Static, Label, Button, LoadingIndicator
from textual.screen import Screen, ModalScreen
from textual.containers import Container, Horizontal, VerticalScroll, Grid



class Confirm_Screen(ModalScreen[bool]):
    CSS_PATH = "css_lib/cofirm_screen.tcss"

    def __init__(self, message:str):
        self.message = message
        super().__init__()

    def compose(self) -> ComposeResult:
        yield Grid(
            Static(self.message, id="question"),
            Button("Cancel", id="cancel", variant="error"),
            Button("OK", id="ok", variant="success")
            ,id="confirmscreen"
        )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "ok":
            self.dismiss(True)
        else:
            self.dismiss(False)


from textual.screen import Screen
from textual.widgets import Static, Input
from textual import events, on, work
from textual.app import ComposeResult
from .confmscn import Confirm_Screen
from .revertpaxmodule import apiPaxFunctions
from .functionsScreen import FunctionsScreen

class Scan_qr(Screen):
    """QR SCANNER"""

    def compose(self) -> ComposeResult:
        yield Static("PlEASE SCAN QR CODE TO BEGIN", classes="question" )
        yield Input(placeholder=">>>>")
    
    @on(Input.Submitted)
    @work
    async def fix_qr(self) -> None:
        self.l = self.query_one(Input).value
        self.disabled = True
        self.serialList = eval(self.l)  # Assuming the QR code contains a list of serial numbers
        if await self.app.push_screen_wait(Confirm_Screen(f"Are these terminals you wish to activate\n{self.serialList}?")):
            self.notify("Activating>>>")

            apifun = apiPaxFunctions() 
            self.thing = await apifun.startPaxGroup(self.serialList)
            thing2 = await apifun.activateTerminals(apifun.idList)
            self.notify(str(thing2))
            self.app.push_screen(FunctionsScreen(self.thing))


import pandas as pd
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Static, Button
from textual import events, on, work
from .DFTable import DataFrameTable
from .revertpaxmodule import apiPaxFunctions, PushConfigs, terminalDetails
from .ti_labels_iccid import create_pdf
from textual.containers import Container, VerticalScroll


class FunctionsScreen(Screen):

    CSS_PATH = "css_lib/group_gunctions.tcss"


    def __init__(self, df:pd.DataFrame) -> None:
        self.df = df
        self.button_list = [
            {'name':'Activate Group','id':'activate','classes':'gtask-buttons'},
            {'name':'Get Terminal Detials','id':'details ','classes':'gtask-buttons'},
            {'name':'Deactivate','id':'deactivate','classes':'gtask-buttons'},
            {'name':'Move Terminals', 'id':'move', 'classes':'gtask-buttons'},
            {'name':'Delete Group', 'id':'delete', 'classes':'gtask-buttons'},
            {'name':'Create Ticket Labels', 'id':'labels', 'classes':'gtask-buttons'}
        ]
        self.op = apiPaxFunctions()
        self.pepe = PushConfigs()
        self.operations = {
            "activate": self.op.activateTerminals,
            "details": terminalDetails,
            "deactivate": self.op.disableTerminals, 
            "move": None,
            "delete": self.op.deleteTerminals,
            "labels": create_pdf,
            "payanywhere": self.pepe.pushPAConfig,
            "broadpos":self.pepe.pushBroadPosEPX,
            "other": self.pepe.pushBroadPos_nonEPX
        }

        
        self.functionDict = {}
        super().__init__()
    
    def compose(self) -> ComposeResult:

        with Container(id="app-grid"):
            with VerticalScroll(id = "top-pane"):
                yield DataFrameTable()
            with VerticalScroll(id = "bottom-left"):
                yield Static("Available Tasks", classes="titleheader")
                yield Button("Activate Group", id="activate", classes="gtask-buttons")
                yield Button("Get Terminal Details", id = "details", classes="gtask-buttons")
                yield Button("Dectivate Group", id="deactivate",classes="gtask-buttons")
                yield Button("Move Terminals", id="move", classes="gtask-buttons")
                yield Button("Delete Group", id ="delete", classes="gtask-buttons")
                yield Button("Create Labels", id="labels", classes="gtask-buttons")
            with VerticalScroll(id= "bottom-right"):
                yield Static("Configuration Tasks", classes="titleheader")
                yield Button("Config for PayAnywhere", id="payanywhere", classes="buttons")
                yield Button("Config for BroadPOS - EPX", id="broadpos",classes="buttons")
                yield Button("Config for BroadPOS - Not EPX", id ="other", classes="buttons")
            
    async def on_mount(self):   
        """if self.df == None:
            self.mdf = pd.read_pickle("/Users/scottjamnik/python-to-google-sheets/E600M04.29.2024.14.051.pkl")
        else: self.mdf = pd.DataFrame(self.df)"""

        self.table = self.query_one(DataFrameTable)
        self.table.add_df(self.df)
        self.app.notify(str(self.df))
        self.df.to_pickle("df.pkl")

    
    @on(Button.Pressed)
    @work
    async def do_stuff(self, event: Button.Pressed):
        operation = self.operations.get(event.button.id)  # type: ignore
        result = await operation(idList = self.df['id'], serialNoList = self.df['serialNo'], df = self.df)  # type: ignore
        self.notify(str(result))
        if event.button.id == "details":
            self.ndf = pd.DataFrame(result)
            self.cdf = pd.concat([self.df,self.ndf], axis=1)
            self.table.update_df(self.cdf)
    
    @on(DataFrameTable.CellSelected)
    def note_cell(self, event:DataFrameTable.CellSelected):
        self.app.notify(str(event.value))


from textual.screen import Screen
from textual.app import App, ComposeResult

from textual.widgets import Static, Input
from textual import events, on, work
import pandas as pd
from .revertpaxmodule import apiPaxFunctions  # Assuming this is in the same directory
from .confmscn import Confirm_Screen  # Assuming this is in the same directory
from .functionsScreen import FunctionsScreen

class Scan_serials(Screen):

    """SERIAL NUMBER INPUT"""

    def __init__(self):

        self.serialNoList = []
        super().__init__()
    
    def compose(self) -> ComposeResult:
        yield Static("PlEASE SCAN OR TYPE SERIAL NUMBER:", classes="question" )
        yield Input(placeholder="S/N")

    @on(Input.Submitted)
    @work
    async def update_serial_list(self):
        user_input = self.query_one(Input)
        serialNo = user_input.value
        self.serialNoList.append(str(serialNo))
        self.mount(Static(serialNo))
        if user_input.value == "0000":
            self.disabled = True
            self.serialNoList.pop()
            #self.notify(str(self.serial_list))
            df = pd.DataFrame({"serialNo":self.serialNoList},dtype='object')
            if await self.app.push_screen_wait(Confirm_Screen(f"Are these terminals you wish to activate\n{self.serialNoList}?")):
                if await self.app.push_screen_wait(Confirm_Screen("Please ensure network connection & open PaxStore on device")):

                    self.notify("Activating>>>")
                    apifun = apiPaxFunctions()
                    self.thing = await apifun.startPaxGroup(self.serialNoList)
                    #thing2 = await apifun.disableTerminals(apifun.idList)
                    #self.notify(str(thing2))
                    #thing3 = await apifun.deleteTerminals(apifun.idList)
                    #self.notify(str(thing3))
                    #thing4 = await apifun.createTerminals(self.serialNoList)
                    #self.notify(str(thing4))
                    self.mdf = pd.DataFrame(self.thing)
                    self.app.push_screen(FunctionsScreen(self.mdf)) # type: ignore
        
        user_input.clear()